import { defineStore } from "pinia";
import type { DamageReportViewModel, UpdateDamageReportViewModel } from "@/viewmodels/admin/unit/damageReport.models";
import { http } from "@/serverCom";
import type { AxiosResponse } from "axios";

export const useDamageReportStore = defineStore("damageReport", {
  state: () => {
    return {
      damageReports: [] as Array<DamageReportViewModel & { tab_pos: number }>,
      totalCount: 0 as number,
      loading: "loading" as "loading" | "fetched" | "failed",
      activeDamageReport: null as string | null,
      activeDamageReportObj: null as DamageReportViewModel | null,
      loadingActive: "loading" as "loading" | "fetched" | "failed",
    };
  },
  actions: {
    formatQueryReturnToPagination(result: AxiosResponse<any, any>, offset: number) {
      this.totalCount = result.data.total;
      result.data.damageReports
        .filter((elem: DamageReportViewModel) => this.damageReports.findIndex((m) => m.id == elem.id) == -1)
        .map((elem: DamageReportViewModel, index: number): DamageReportViewModel & { tab_pos: number } => {
          return {
            ...elem,
            tab_pos: index + offset,
          };
        })
        .forEach((elem: DamageReportViewModel & { tab_pos: number }) => {
          this.damageReports.push(elem);
        });
    },
    fetchOpenDamageReports(offset = 0, count = 25, search = "", clear = false) {
      if (clear) this.damageReports = [];
      this.loading = "loading";
      http
        .get(`/admin/damageReport?done=false&offset=${offset}&count=${count}${search != "" ? "&search=" + search : ""}`)
        .then((result) => {
          this.formatQueryReturnToPagination(result, offset);
          this.loading = "fetched";
        })
        .catch((err) => {
          this.loading = "failed";
        });
    },
    fetchDoneDamageReports(offset = 0, count = 25, search = "", clear = false) {
      if (clear) this.damageReports = [];
      this.loading = "loading";
      http
        .get(`/admin/damageReport?done=true&offset=${offset}&count=${count}${search != "" ? "&search=" + search : ""}`)
        .then((result) => {
          this.formatQueryReturnToPagination(result, offset);
          this.loading = "fetched";
        })
        .catch((err) => {
          this.loading = "failed";
        });
    },
    async getAllDamageReports(): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/damageReport?noLimit=true`).then((res) => {
        return { ...res, data: res.data.damageReports };
      });
    },
    async getDamageReportsByIds(ids: Array<string>): Promise<AxiosResponse<any, any>> {
      return await http
        .post(`/admin/damageReport/ids`, {
          ids,
        })
        .then((res) => {
          return { ...res, data: res.data.damageReports };
        });
    },
    async searchDamageReports(search: string): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/damageReport?search=${search}&noLimit=true`).then((res) => {
        return { ...res, data: res.data.damageReports };
      });
    },
    async getAllDamageReportsWithRelated(
      related: "vehicle" | "equipment" | "wearable",
      relatedId: string
    ): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/damageReport/${related}/${relatedId}?noLimit=true`).then((res) => {
        return { ...res, data: res.data.damageReports };
      });
    },
    async searchDamageReportsWithRelated(
      related: "vehicle" | "equipment" | "wearable",
      relatedId: string,
      search: string
    ): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/damageReport/${related}/${relatedId}?search=${search}&noLimit=true`).then((res) => {
        return { ...res, data: res.data.damageReports };
      });
    },
    fetchDamageReportByActiveId() {
      this.loadingActive = "loading";
      http
        .get(`/admin/damageReport/${this.activeDamageReport}`)
        .then((res) => {
          this.activeDamageReportObj = res.data;
          this.loadingActive = "fetched";
        })
        .catch((err) => {
          this.loadingActive = "failed";
        });
    },
    fetchDamageReportById(id: string) {
      return http.get(`/admin/damageReport/${id}`);
    },
    loadDamageReportImage(url: string) {
      return http.get(`/admin/damageReport/${this.activeDamageReportObj?.id}/${url}`, {
        responseType: "blob",
      });
    },
    async updateDamageReport(damageReport: UpdateDamageReportViewModel): Promise<AxiosResponse<any, any>> {
      const result = await http.patch(`/admin/damageReport/${damageReport.id}`, {
        status: damageReport.status,
        noteByWorker: damageReport.noteByWorker,
        done: damageReport.done,
      });
      return result;
    },
  },
});
