import { defineStore } from "pinia";
import { http } from "@/serverCom";
import { useEquipmentStore } from "./equipment";
import type { RepairViewModel } from "@/viewmodels/admin/unit/repair.models";

export const useEquipmentRepairStore = defineStore("equipmentRepair", {
  state: () => {
    return {
      repairs: [] as Array<RepairViewModel & { tab_pos: number }>,
      totalCount: 0 as number,
      loading: "loading" as "loading" | "fetched" | "failed",
    };
  },
  actions: {
    fetchRepairForEquipment(offset = 0, count = 25, search = "", clear = false) {
      const equipmentId = useEquipmentStore().activeEquipment;
      if (clear) this.repairs = [];
      this.loading = "loading";
      http
        .get(
          `/admin/repair/equipment/${equipmentId}?offset=${offset}&count=${count}${search != "" ? "&search=" + search : ""}`
        )
        .then((result) => {
          this.totalCount = result.data.total;
          result.data.repairs
            .filter((elem: RepairViewModel) => this.repairs.findIndex((m) => m.id == elem.id) == -1)
            .map((elem: RepairViewModel, index: number): RepairViewModel & { tab_pos: number } => {
              return {
                ...elem,
                tab_pos: index + offset,
              };
            })
            .forEach((elem: RepairViewModel & { tab_pos: number }) => {
              this.repairs.push(elem);
            });
          this.loading = "fetched";
        })
        .catch((err) => {
          this.loading = "failed";
        });
    },
  },
});
