import { defineStore } from "pinia";
import { http } from "@/serverCom";
import type { InspectionPlanViewModel } from "@/viewmodels/admin/unit/inspection/inspectionPlan.models";
import { useVehicleTypeStore } from "./vehicleType";

export const useVehicleTypeInspectionPlanStore = defineStore("vehicleTypeInspectionPlan", {
  state: () => {
    return {
      inspectionPlans: [] as Array<InspectionPlanViewModel>,
      totalCount: 0 as number,
      loading: "loading" as "loading" | "fetched" | "failed",
    };
  },
  actions: {
    fetchInspectionPlanForVehicleType() {
      const vehicleTypeId = useVehicleTypeStore().activeVehicleType;
      this.loading = "loading";
      http
        .get(`/admin/inspectionPlan/vehicleType/${vehicleTypeId}?noLimit=true`)
        .then((result) => {
          this.inspectionPlans = result.data.inspectionPlans;
          this.loading = "fetched";
        })
        .catch((err) => {
          this.loading = "failed";
        });
    },
  },
});
