import { defineStore } from "pinia";
import type {
  WearableTypeViewModel,
  CreateWearableTypeViewModel,
  UpdateWearableTypeViewModel,
} from "@/viewmodels/admin/unit/wearable/wearableType.models";
import { http } from "@/serverCom";
import type { AxiosResponse } from "axios";

export const useWearableTypeStore = defineStore("wearableType", {
  state: () => {
    return {
      wearableTypes: [] as Array<WearableTypeViewModel & { tab_pos: number }>,
      totalCount: 0 as number,
      loading: "loading" as "loading" | "fetched" | "failed",
      activeWearableType: null as string | null,
      activeWearableTypeObj: null as WearableTypeViewModel | null,
      loadingActive: "loading" as "loading" | "fetched" | "failed",
    };
  },
  actions: {
    fetchWearableTypes(offset = 0, count = 25, search = "", clear = false) {
      if (clear) this.wearableTypes = [];
      this.loading = "loading";
      http
        .get(`/admin/wearableType?offset=${offset}&count=${count}${search != "" ? "&search=" + search : ""}`)
        .then((result) => {
          this.totalCount = result.data.total;
          result.data.wearableTypes
            .filter((elem: WearableTypeViewModel) => this.wearableTypes.findIndex((m) => m.id == elem.id) == -1)
            .map((elem: WearableTypeViewModel, index: number): WearableTypeViewModel & { tab_pos: number } => {
              return {
                ...elem,
                tab_pos: index + offset,
              };
            })
            .forEach((elem: WearableTypeViewModel & { tab_pos: number }) => {
              this.wearableTypes.push(elem);
            });
          this.loading = "fetched";
        })
        .catch((err) => {
          this.loading = "failed";
        });
    },
    async getAllWearableTypes(): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/wearableType?noLimit=true`).then((res) => {
        return { ...res, data: res.data.wearableTypes };
      });
    },
    async searchWearableTypes(search: string): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/wearableType?search=${search}&noLimit=true`).then((res) => {
        return { ...res, data: res.data.wearableTypes };
      });
    },
    fetchWearableTypeByActiveId() {
      this.loadingActive = "loading";
      http
        .get(`/admin/wearableType/${this.activeWearableType}`)
        .then((res) => {
          this.activeWearableTypeObj = res.data;
          this.loadingActive = "fetched";
        })
        .catch((err) => {
          this.loadingActive = "failed";
        });
    },
    fetchWearableTypeById(id: string) {
      return http.get(`/admin/wearableType/${id}`);
    },
    async createWearableType(wearableType: CreateWearableTypeViewModel): Promise<AxiosResponse<any, any>> {
      const result = await http.post(`/admin/wearableType`, {
        type: wearableType.type,
        description: wearableType.description,
      });
      this.fetchWearableTypes();
      return result;
    },
    async updateWearableType(wearableType: UpdateWearableTypeViewModel): Promise<AxiosResponse<any, any>> {
      const result = await http.patch(`/admin/wearableType/${wearableType.id}`, {
        type: wearableType.type,
        description: wearableType.description,
      });
      this.fetchWearableTypes();
      return result;
    },
    async deleteWearableType(wearableType: number): Promise<AxiosResponse<any, any>> {
      const result = await http.delete(`/admin/wearableType/${wearableType}`);
      this.fetchWearableTypes();
      return result;
    },
  },
});
